//
//  QLookPlugIn.m
//  QLook
//
//  Created by Dustin O'Connor on 3/24/12.
//  Copyright (c) 2012 d0cut0uch. All rights reserved.
//

// It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering
#import <OpenGL/CGLMacro.h>

#import "QLookPlugIn.h"
#import "ArgumentKeys.h"

#define	kQCPlugIn_Name				@"QLook"
#define	kQCPlugIn_Description		@"QLook description"



@implementation QLookPlugIn

// Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputPath, outputImage;

+ (NSDictionary *)attributes
{
	// Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary *)attributesForPropertyPortWithKey:(NSString *)key
{

	return nil;
}

+ (QCPlugInExecutionMode)executionMode
{
	// Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode)timeMode
{
	// Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	return kQCPlugInTimeModeIdle;
}

- (id)init
{
	self = [super init];
	if (self) {
			}
	
	return self;
}


@end

@implementation QLookPlugIn (Execution)

static void _BufferReleaseCallback(const void* address, void* context)
{
	/* Destroy the CGContext and its backing */
	free((void*)address);
}


- (BOOL)startExecution:(id <QCPlugInContext>)context
{
    
    
   
	
	return YES;
}

static void _TextureReleaseCallback(CGLContextObj cgl_ctx, GLuint name, void* context)
{
	/* Delete the OpenGL texture */
	glDeleteTextures(1, &name);
}

- (void)enableExecution:(id <QCPlugInContext>)context
{
	// Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
}

- (BOOL)execute:(id <QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary *)arguments
{
	GLuint name = [[arguments objectForKey:kArgumentKey_TextureName] unsignedIntValue];
	CGLContextObj cgl_ctx = [context CGLContextObj];
	id	provider;
 
    /////
    CGContextRef                    bitmapContext;
	CGImageRef                       image;
	void*                                   baseAddress;
	size_t                                  rowBytes;
	NSString*                           path;
	CGRect                              bounds;
    
    path =  self.inputPath;
    
    
    NSURL *fileURL = [NSURL fileURLWithPath:path];
    
    NSDictionary *dict = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:self.inputIcon] 
                                                     forKey:(NSString *)kQLThumbnailOptionIconModeKey];
    CGImageRef ref = QLThumbnailImageCreate(kCFAllocatorDefault, 
                                            (CFURLRef)fileURL, 
                                            CGSizeMake((CGFloat)self.inputWidth, (CGFloat)self.inputHeight),
                                            (CFDictionaryRef)dict);
    
    
      if (ref != NULL) 
      {
          image = ref; 
      }else {
          
      NSImage *icon = [[NSWorkspace sharedWorkspace] iconForFile:path];
          if(icon){
        [icon setSize:CGSizeMake((CGFloat)self.inputWidth, (CGFloat)self.inputHeight)];
              CGImageSourceRef source;
               source = CGImageSourceCreateWithData((CFDataRef)[icon TIFFRepresentation], NULL);
              CGImageRef icoRef =  CGImageSourceCreateImageAtIndex(source, 0, NULL);
              image = icoRef;
          }
      }
    
    
    
	//CFRelease(source);
	if(image == NULL)
        return NO;
    
    /* Create CGContext backing */
	rowBytes = CGImageGetWidth(image) * 4;
	if(rowBytes % 16)
        rowBytes = ((rowBytes / 16) + 1) * 16;
	baseAddress = valloc(CGImageGetHeight(image) * rowBytes);
	if(baseAddress == NULL) {
		CGImageRelease(image);
		return NO;
	}
	
	/* Create CGContext and draw image into it */
	bitmapContext = CGBitmapContextCreate(baseAddress, CGImageGetWidth(image), CGImageGetHeight(image), 8, rowBytes, [context colorSpace], kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Host);
	if(bitmapContext == NULL) {
		free(baseAddress);
		CGImageRelease(image);
		return NO;
	}
	bounds = CGRectMake(0, 0, CGImageGetWidth(image), CGImageGetHeight(image));
	CGContextClearRect(bitmapContext, bounds);
	CGContextDrawImage(bitmapContext, bounds, image);
	
	/* We don't need the image and context anymore */
	CGImageRelease(image);
	CGContextRelease(bitmapContext);
	
	/* Create image provider from backing */
#if __BIG_ENDIAN__
	_placeHolderProvider = [[context outputImageProviderFromBufferWithPixelFormat:QCPlugInPixelFormatARGB8 pixelsWide:CGImageGetWidth(image) pixelsHigh:CGImageGetHeight(image) baseAddress:baseAddress bytesPerRow:rowBytes releaseCallback:_BufferReleaseCallback releaseContext:NULL colorSpace:[context colorSpace] shouldColorMatch:YES] retain];
#else
	_placeHolderProvider = [[context outputImageProviderFromBufferWithPixelFormat:QCPlugInPixelFormatBGRA8 pixelsWide:CGImageGetWidth(image) pixelsHigh:CGImageGetHeight(image) baseAddress:baseAddress bytesPerRow:rowBytes releaseCallback:_BufferReleaseCallback releaseContext:NULL colorSpace:[context colorSpace] shouldColorMatch:YES] retain];
#endif
	if(_placeHolderProvider == nil) {
		free(baseAddress);
		return NO;
	}
	

	/* Create a image provider from the OpenGL texture passed through the execution arguments if any - The provider acquires ownership of the texture */
	if(name) {
		provider = [context outputImageProviderFromTextureWithPixelFormat:[arguments objectForKey:kArgumentKey_TextureFormat] pixelsWide:[[arguments objectForKey:kArgumentKey_TextureWidth] unsignedIntValue] pixelsHigh:[[arguments objectForKey:kArgumentKey_TextureHeight] unsignedIntValue] name:name flipped:YES releaseCallback:_TextureReleaseCallback releaseContext:NULL colorSpace:(CGColorSpaceRef)[arguments objectForKey:kArgumentKey_TextureColorSpace] shouldColorMatch:YES];
		if(provider == nil) {
			glDeleteTextures(1, &name);
			return NO;
		}
	}
	
	/* Set output image */
	self.outputImage = (name ? provider : _placeHolderProvider);
	
	return YES;
}

- (void)disableExecution:(id <QCPlugInContext>)context
{
	// Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
}

- (void)stopExecution:(id <QCPlugInContext>)context
{
	/* Our instance is about to stop running */
	[[context userInfo] removeObjectForKey:kArgumentKeyPrefix];
	
	/* Release placeholder image provider */
	[_placeHolderProvider release];

}

@end
